// I2C・SPI用
#include <Wire.h>
#include <SPI.h>

// 画面用
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET     4
#define SCREEN_ADDRESS 0x3C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
#define NUMFLAKES     10
#define LOGO_HEIGHT   16
#define LOGO_WIDTH    16

// NFCリーダー用
#include "Adafruit_PN532.h"
#define PN532_IRQ   (0)
#define PN532_RESET (0)  // Not connected
Adafruit_PN532 nfc(PN532_IRQ, PN532_RESET);

//  ftDuino用
#include <FtduinoSimple.h>

// タグ情報
uint8_t ID1[] = {0x27, 0x86, 0x88, 0x03};
uint8_t ID2[] = {0x0F, 0x3A, 0x88, 0x03};
uint8_t ID3[] = {0xDC, 0x5A, 0x18, 0x4A};
uint8_t ID4[] = {0xFF, 0xF4, 0x6D, 0x03};
uint8_t ID5[] = {0x5D, 0xAA, 0x6D, 0x03};


void setup(void) {

  Serial.begin(9600);
  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;); // Don't proceed, loop forever
  }

  display.clearDisplay();  //  バッファを空にする
  display.setTextSize(2); // テキストサイズの設定
  display.setTextColor(SSD1306_WHITE);  //  文字の色
  display.setCursor(10, 0);  //  カーソルの位置
  display.println(F(" Waiting "));  // Flush ram から直接読む
  display.display();      // 画面の更新

  Serial.begin(115200);
  // USBを最大3秒待つ
  uint32_t to = millis();
  while ((!Serial) && ((millis() - to) < 3000))
    delay(10);

  nfc.begin();

  uint32_t versiondata = nfc.getFirmwareVersion();  //　ファームウェアのバージョン情報を取得
  // バージョン情報を取得できない場合
  if (! versiondata) {
    Serial.print("== NFCリーダーが見つかりません ==");
    while (1); //  ストップ
  }

  // バージョン情報を表示
  Serial.print("Found chip PN5"); Serial.println((versiondata >> 24) & 0xFF, HEX);
  Serial.print("Firmware ver. "); Serial.print((versiondata >> 16) & 0xFF, DEC);
  Serial.print('.'); Serial.println((versiondata >> 8) & 0xFF, DEC);

  nfc.SAMConfig();

  Serial.println("待機中 ... ");

  // 内部LED  ON
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop(void) {
  static uint8_t registered_uid[4];  //  UID Valueを格納
  static bool registered = false;

  uint8_t uid[] = { 0, 0, 0, 0, 0, 0, 0 };  // 返されたUIDを格納するためのバッファ
  uint8_t uidLength;                        // UIDの長さ（ISO14443Aカードのタイプに応じて4バイトまたは7バイト）

  // ISO14443Aタイプのカード（Mifareなど）を待ちます。
  // 'uid' には UID Valueが入ります 16進数
  // uidLength には UID Lengthが入ります (バイト数)
  if (nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength, 100)) {
    // カードの基本情報を表示
    Serial.println("Found an ISO14443A card");
    Serial.print("  UID Length: "); Serial.print(uidLength, DEC); Serial.println(" bytes");
    Serial.print("  UID Value: ");  nfc.PrintHex(uid, uidLength);
    Serial.println("");
    memcpy(registered_uid, uid, uidLength);

    if (uidLength == 4) {
      Serial.println("Mifare Classic card (4 byte UID)");
      Serial.println("マイフェア クラシックカードです");


      if (memcmp(registered_uid, ID1, uidLength) == 0) {
        Serial.println("カードを受け付けました");
        Serial.println("==  ABE ==");
        // 名前の表示
        display.clearDisplay();  //  バッファを空にする
        display.setCursor(10, 0);  //  カーソルの位置
        display.println("ABE");
        display.display();      // 画面の更新
      }
      if (memcmp(registered_uid, ID2, uidLength) == 0) {
        Serial.println("カードを受け付けました");
        Serial.println("==  KATOU ==");
        // 名前の表示
        display.clearDisplay();  //  バッファを空にする
        display.setCursor(10, 0);  //  カーソルの位置
        display.println("KATOU");
        display.display();      // 画面の更新
      }
      if (memcmp(registered_uid, ID3, uidLength) == 0) {
        Serial.println("カードを受け付けました");
        Serial.println("==  SAITOU ==");
        // 名前の表示
        display.clearDisplay();  //  バッファを空にする
        display.setCursor(10, 0);  //  カーソルの位置
        display.println("SAITOU");
        display.display();      // 画面の更新
      }

      if (memcmp(registered_uid, ID4, uidLength) == 0) {
        Serial.println("カードを受け付けました");
        Serial.println("==  YAMADA ==");
        // 名前の表示
        display.clearDisplay();  //  バッファを空にする
        display.setCursor(10, 0);  //  カーソルの位置
        display.println("YAMADA");
        display.display();      // 画面の更新
      }

      if (memcmp(registered_uid, ID5, uidLength) == 0) {
        Serial.println("カードを受け付けました");
        Serial.println("==  MIYAMOTO ==");
        // 名前の表示
        display.clearDisplay();  //  バッファを空にする
        display.setCursor(10, 0);  //  カーソルの位置
        display.println("MIYAMOTO");
        display.display();      // 画面の更新
      }
    }
  } 

}
